<?php
/**
 * ArchivDB.class.php
 *
 * Diese Datei enthlt die Datenbankfunktionen des Plugins
 * Dieses Plugin basiert auf PHP5 und steht unter der GPL.
 *
 * @author  	Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @package 	ArchivierungsPlugin
 * @copyright 	IBIT 2010
 * @version 	1.2
 */

//Imports
require_once "lib/functions.php";
require_once 'lib/archiv.inc.php';
require_once 'lib/datei.inc.php';

/**
 * Statische DB-Methoden fr das Archivieren
 *
 */
class ArchivDB
{
	/**
	 * Holt eine Liste aller Institute, gruppiert nach Fakultten aus der DB
	 * und gibt sie als array zurck
	 *
	 * @return array
	 *
	 */
	public function getInstitutesByFaculty()
	{
		$db=new DB_Seminar();
		$db2=new DB_Seminar();

		$result = array();
		// Fakultten aus der DB holen
		$db->query("SELECT Institut_id, Name, 1 AS is_fak  FROM Institute WHERE Institut_id=fakultaets_id ORDER BY Name");
		while ($db->next_record())
		{
			#printf ("<option value=\"%s\" style=\"%s\">%s </option>\n", $db->f("Institut_id"),($db->f("is_fak") ? "font-weight:bold;" : ""), htmlReady(substr($db->f("Name"), 0, 70)));
			if ($db->f("is_fak"))
			{
				$db2->query("SELECT Institut_id, Name FROM Institute WHERE fakultaets_id='" .$db->f("Institut_id") . "' AND institut_id!='" .$db->f("Institut_id") . "' ORDER BY Name");
				$institutes = array();
				while ($db2->next_record())
				{
					$inst = array(
						'id' => $db2->f("Institut_id"),
						'name' => $db2->f("Name")
					);
					array_push($institutes, $inst);
				}
			}
			$fak = array(
				'id' => $db->f("Institut_id"),
				'name' =>  $db->f("Name"),
				'is_fak' => ($db->f("is_fak") ? true : false),
				'institutes' => $institutes
			);
			array_push($result, $fak);
		}
		return $result;
	}

    /**
     * Gibt den Namen eines Institutes zu einer Id zurck
     *
     * @param string $id
     * @return string Name
     */
    public static function getInstitutName($id)
    {
        return DBManager::get()->query("SELECT Name FROM Institute WHERE Institut_id='{$id}'")->fetchColumn();
    }

	/**
	 * Enter description here...
	 *
	 * @param string $institut_id
	 * @param string $semester_id
	 * @param unknown_type $lastChange
	 * @return array()
	 */
	public static function getCourses($institut_id, $semester_id, $lastChange, $duration = '0', $semclass, $prefix = '',$only_this_dozent='',$days_offline=0)
	{
	    if(empty($semclass)){
	        return array();
	    }

	    //keine semester ausgewhlt, also aktuelles whlen
		if ($semester_id == '0' || empty($semester_id))
		{
		    $semester_id = DBManager::get()->query("SELECT semester_id FROM semester_data WHERE UNIX_TIMESTAMP() BETWEEN beginn AND ende;")->fetch(PDO::FETCH_COLUMN);
		}

		//semesterinfos holen
		$semester_end = DBManager::get()->query("SELECT ende FROM semester_data WHERE semester_id='{$semester_id}'")->fetch(PDO::FETCH_COLUMN);

		$dozenten = array();
		if ($only_this_dozent) {
		    $dozenten[] = $only_this_dozent;
		}
		if ($days_offline) {
		    $query = "SELECT user_id
            FROM auth_user_md5
            LEFT JOIN user_data ON user_id = sid
            WHERE perms = 'dozent' AND
            IFNULL(changed,0) < DATE_SUB(CURDATE(),INTERVAL ".(int)$days_offline." DAY) ";
		   $rs = DBManager::get()->query($query);
		    while($dozent = $rs->fetchColumn()) {
		        $dozenten[] = $dozent;
		    }
		}

		//institute bzw. fakultten auswhlen
		$result = array();

		// alle
		if(!empty($institut_id) && $institut_id == "0" )
		{
			$result = self::selectCourses($semester_end, '', $lastChange, $semclass,$prefix,$dozenten);
		}
		// nur bestimmte
		else
		{
			// fakultt
			$db = new DB_Seminar;
			$db->queryf("SELECT Institut_id FROM Institute WHERE fakultaets_id = '%s' ORDER BY Name", $institut_id);
			if($db->num_rows() > 0)
			{
				while ($db->next_record())
				{
					$fak_result = self::selectCourses($semester_end, $db->f("Institut_id"), $lastChange, $duration,$semclass, $prefix,$dozenten);
					$result = array_merge($result, $fak_result);
				}
			}
			// institut
			else
			{
				$result = self::selectCourses($semester_end, $institut_id, $lastChange, $duration,$semclass,$prefix,$dozenten);
			}
		}

		return $result;
	}

	/**
	 * Enter description here...
	 *
	 * @param string $semester_end
	 * @param string $institut_id
	 * @param string $lastChange
	 * @return array
	 */
	private static function selectCourses($semester_end, $institut_id, $lastChange, $duration, $semclass, $prefix, $dozenten = array())
	{
		$db = new DB_Seminar;
		if(!empty($institut_id)) { //institutsauswahl
			$inst_condition = "AND s.Institut_id='$institut_id'";
		} else {
			$inst_condition = "";
		}
		if($duration != '-1') { // seminardauer auswahl
		    $duration = 'AND s.duration_time != -1';
		} else {
		    $duration = '';
		}

		//und hier filter nach veranstaltungstyp
		$sem_types = array();
		foreach($semclass as $class) $sem_types = array_merge($sem_types, array_keys(SeminarCategories::Get($class)->getTypes()));
        $sem_type_condition = " AND s.status IN ('" . join("','", $sem_types) . "') ";

        if ($prefix) {
        	$prefix_condition = " AND s.Name LIKE '" . addslashes($prefix) . "%' ";
        } else {
        	$prefix_condition = "";
        }


        $courses = array();

		//kurse aus der DB holen ACHTUNG, kann lange dauern...
		$db->queryf("SELECT s.Seminar_id, s.Name, s.start_time, s.duration_time,
					s.status, i.Name as inst_name, sd.name as sem_name,
					GROUP_CONCAT(su.user_id) as dozenten
		             FROM seminare s
		             LEFT JOIN Institute i ON s.Institut_id=i.Institut_id
		             LEFT JOIN semester_data sd ON s.start_time BETWEEN sd.beginn AND sd.ende
		             LEFT JOIN seminar_user su ON su.Seminar_id = s.Seminar_id AND su.status='dozent'
		             WHERE s.start_time <= %s %s %s %s %s
		             GROUP BY s.Seminar_id
		             ORDER BY s.start_time, s.Name",
		          $semester_end,
		          $inst_condition,
		          $sem_type_condition,
		          $prefix_condition,
		          $duration);
		if($db->num_rows() > 0)
		{
			while ($db->next_record())
			{
			    $c_dozenten = explode(',' , $db->f("dozenten"));
			    if (count($dozenten) && !count(array_intersect($dozenten, $c_dozenten))) {
			        continue;
			    }
				//wenn letzt aktivitt lter als angegebener zeitraum, dann hinzufgen
				$lastActivity = lastActivity($db->f("Seminar_id"));
				if ($lastChange === false || ($lastActivity < $lastChange))
				{
					//kurs vorhanden...
					$course = array(
						"seminar_id" => $db->f("Seminar_id"),
						"name" => $db->f("Name"),
					    "duration" => ($db->f("duration_time")=='-1')?'(dauerhafte Veranstaltung)':'',
						"last_activity" => $lastActivity,
						"semester" => $db->f('sem_name') ? $db->f('sem_name') : "Semester unbekannt!",
						"institute" => $db->f('inst_name'),
					    "type" => $db->f("status"),
					    "dozenten" => $c_dozenten
						);
					array_push($courses, $course);
				}
			}
		}
		//echo "<br>selectCourses<br><hr><br>";
		//print_r($courses);
		return $courses;
	}

	/**
	 * Gibt ein Array mit allen archivierten Veranstaltungen, gruppiert nach Semester, sortiert nach Name zuurck
	 *
	 * @return array()
	 */
	public static function getAllArchivedCourses()
	{
		$list = array();
		//Daten aus der Datenbank holen
		//1. semster holen.
		$db = new DB_Seminar;
		$db->query("SELECT DISTINCT semester FROM archiv ORDER BY start_time DESC");
		while ($db->next_record())
		{
			$courses = array();
			$db2 = new DB_Seminar;
			$db2->queryf("SELECT name, dozenten, institute, seminar_id, archiv_file_id, forumdump, wikidump, mkdate FROM archiv WHERE semester = '%s' ORDER BY institute, name", $db->f("semester"));
			while ($db2->next_record())
			{
				$course = array();
				$course['name'] = $db2->f('name');
				$course['dozenten'] = $db2->f('dozenten');
				$course['einrichtung'] = $db2->f('institute');
				$course['dump_id'] = $db2->f('seminar_id');
				$course['date'] = date("d.m.y", $db2->f('mkdate'));
				if($db2->f('archiv_file_id'))
				{
					$course['file_link'] = GetDownloadLink($db2->f('archiv_file_id'), 'Dateisammlung'.'_'.str_replace(' ', '_', substr($db2->f("name"),0,200)).'.zip', 1);
				}
				if($db2->f('wikidump') != '')
				{
					#echo "wiki: '".$db2->f('wikidump')."'<br/>";
					$course['wiki'] = true;
				}
				if($db2->f('forumdump') != '')
				{
					#echo "wiki: '".$db2->f('forumdump')."'<br/>";
					$course['forum'] = true;
				}
				array_push($courses, $course);
			}
			array_push($list, array('semester' => $db->f("semester"), 'courses' => $courses, 'count' => $db2->affected_rows()));
		}
		return $list;
	}

	/**
	 * Holt das Archiv fr ein Semester aus der DB
	 *
	 * @param string $semester
	 * @return array()
	 */
	public static function getArchivedCourses($semester)
	{
		//Daten aus der Datenbank holen
		$courses = array();
		$db2 = new DB_Seminar;
		$db2->queryf("SELECT name, dozenten, institute, seminar_id, archiv_file_id, forumdump, wikidump, mkdate FROM archiv WHERE semester = '%s' ORDER BY institute, name", addslashes($semester));
		while ($db2->next_record())
		{
			$course = array();
			$course['name'] = $db2->f('name');
			$course['dozenten'] = $db2->f('dozenten');
			$course['einrichtung'] = $db2->f('institute');
			$course['dump_id'] = $db2->f('seminar_id');
			$course['date'] = date("d.m.y", $db2->f('mkdate'));
			if($db2->f('archiv_file_id'))
			{
				$course['file_link'] = GetDownloadLink($db2->f('archiv_file_id'), 'Dateisammlung'.'_'.str_replace(' ', '_', substr($db2->f("name"),0,200)).'.zip', 1);
			}
			if($db2->f('wikidump') != '')
			{
				#echo "wiki: '".$db2->f('wikidump')."'<br/>";
				$course['wiki'] = true;
			}
			if($db2->f('forumdump') != '')
			{
				#echo "wiki: '".$db2->f('forumdump')."'<br/>";
				$course['forum'] = true;
			}
			$courses[] = $course;
		}
		return array('semester' => $semester, 'courses' => $courses, 'count' => $db2->affected_rows());
	}

    /**
     * Gibt eine Liste aller semester mit archivierten Veranstaltungen zurck
     *
     * @return array()
     */
    public static function getArchivedSemesters()
    {
        return DBManager::Get()->query("SELECT DISTINCT semester FROM archiv ORDER BY semester;")->fetchAll(PDO::FETCH_ASSOC);
    }

	/**
	 * berprft die Datenbank auf kaputte Veranstaltungen und gibt eine Liste
	 * aller defekten Kurse zurck.
	 *
	 * @return array $courses
	 */
	public static function getBrokenCourses()
	{
		$courses = array();
		$db = new DB_Seminar;
		$db->query("SELECT s.Seminar_id AS id, s.Name AS name FROM archiv AS a, seminare AS s WHERE a.seminar_id = s.Seminar_id;");
		while($db->next_record())
		{
			$course = array(
				'id' => $db->f("id"),
				'name' => $db->f("name"),
				'problem' => 'seminare/archiv'
			);
			array_push($courses, $course);
		}
		$db->query("SELECT DISTINCT s.Seminar_id AS id, s.Name AS name FROM archiv_user AS au, seminare AS s WHERE au.seminar_id = s.Seminar_id;");
		while($db->next_record())
		{
			foreach ($courses as $index=>$exist)
			{
				if($exist['id'] == $db->f("id"))
				{
					$courses[$index]['problem'] .= 'und seminare/archiv_user';
					$edit = true;
				}
			}
			if ($edit != true)
			{
				$course = array(
					'id' => $db->f("id"),
					'name' => $db->f("name"),
					'problem' => 'seminare/archiv_user'
				);
				array_push($courses, $course);
			}
		}
		return $courses;
	}

	/**
	 * Enter description here...
	 *
	 * @param array $courses
	 * @return unknown
	 */
	public static function doArchiveCourses($courses, $semester_id, $do_not_archive = false)
	{
	    set_time_limit(0);
		ob_start();
		//OL-Anpassung: Gibt es die Module-Tabelle?
		$module_db = DBManager::get()->query("SHOW TABLE STATUS LIKE 'module'")->fetch(PDO::FETCH_ASSOC);

		foreach ($courses as $index=>$s_id)
		{
			$archiv = array();
			if ($do_not_archive == false) {
				//OL-Anpassung fr module
				if (!empty($module_db)) {
					//NEU, modulzugehrigkeit mit archivieren.
					$semtree_ids = DBManager::get()->query("SELECT sem_tree_id FROM seminar_sem_tree WHERE seminar_id = '{$s_id}'")->fetchAll(PDO::FETCH_COLUMN);
					if (count($semtree_ids) > 0) {
						foreach ($semtree_ids as $semtree_id)
						{
							$kon_ids = DBManager::get()->query("SELECT m.modul_kon_id FROM module m LEFT JOIN mod_zuordnung mz ON m.modul_abst_id = mz.modul_abst_id WHERE mz.sem_tree_id = '{$semtree_id}' AND m.semester = '{$semester_id}'")->fetchAll(PDO::FETCH_COLUMN);
						}
						if (count($kon_ids) > 0) {
							foreach ($kon_ids as $kon_id)
							{
								DBManager::get()->exec("INSERT INTO archiv_module (seminar_id, Modul_kon_id) VALUES ('{$s_id}', '{$kon_id}')");
							}
						}
					}
				}
			 in_archiv($s_id);
			}
		$course = array();
        try {
			$sem = new Seminar($s_id);
			$course['name'] = $sem->getName();
	        // Delete that Seminar.
	        $sem->delete();
	        if ($messages = $sem->getStackedMessages()) {
	            foreach ($messages as $type => $message_data) {
	                $course['messages'] = array_merge((array)$course['messages'], (array)$message_data['details']);
	            }
	        }
        } catch (Exception $e) {
        	$course['error'] = $e->getMessage();
        }

?>
	<tr class="<?=($index%2==0)?'steel1':'steelgraulight' ?>">
		<td valign="top" align="right"><?=$index+1 ?>)</td>
		<td>
		<?=htmlready($course['name']) ?>
		<ul>
		<?foreach($course['messages'] as $m) echo '<li>'.$m.'</li>'; ?>
		</ul>
		<? if ($do_not_archive) : ?>
		  <? if(!$course['error']): ?><b>Veranstaltung erfolgreich gelscht</b>
          <? else: ?><b style="color:#FF0000">Fehler: Veranstaltung wurde nicht oder nicht vollstndig gelscht (<?=htmlready($course['error'])?>)</b><? endif; ?>
		<? else :?>
			<? if(!$course['error']): ?><b>Veranstaltung erfolgreich archiviert</b>
			<? else: ?><b style="color:#FF0000">Fehler: Veranstaltung wurde zwar archiviert, aber nicht gelscht (<?=htmlready($course['error'])?>)</b><? endif; ?>
		<? endif;?>
		</td>
		<td align="right" valign="top">
		<? if ($do_not_archive) : ?>
		  &nbsp;
		<? else :?>
		  <a target="_blank" href="<?=UrlHelper::getLink('archiv.php?dump_id=' . $s_id) ?>"><img src="<?=Assets::url() ?>images/icons/16/blue/info.png" border="0" alt="Komplettansicht" title="Komplettansicht" /></a>
		  <a href="<?=UrlHelper::getLink('archiv.php?delete_id=' . $s_id) ?>"><img src="<?=Assets::url() ?>images/icons/16/blue/trash.png" border="0" alt="Diese Veranstaltung aus dem Archiv entfernen" title="Diese Veranstaltung aus dem Archiv entfernen" /></a>
		<? endif;?>
		</td>
	</tr>
<?
		}
		return ob_get_clean();
	}

    /**
     * Schtzt Veranstaltungen vor dem Archivieren, in dem es das Datum der letzten nderung aktualisiert
     *
     * @param array() $courses
     * @return integer Anzahl Veranstaltungen
     */
    public static function protectCourses($courses)
    {
        foreach ($courses as $seminar_id)
        {
            DBManager::get()->exec("UPDATE seminare SET chdate=UNIX_TIMESTAMP() WHERE Seminar_id='{$seminar_id}'");
        }
        return count($courses);
    }

	/**
	 * Lscht archivierte Veranstaltungen
	 *
	 * @param array $courses
	 */
	public static function deleteCourses($courses)
	{
		#print_r($courses);

		$result = array();
		$db = new DB_Seminar;
		foreach ($courses as $delete_id)
		{
			$info = array();
			//nachgucken, ob archiv vorhanden ist, wenn ja lschen
			$db->query("SELECT name,archiv_file_id,semester FROM archiv WHERE seminar_id='$delete_id';");
			if($db->next_record())
			{
				$info['name'] = $db->f("name");
				$db2 = new DB_Seminar;
				$db2->query("DELETE FROM archiv WHERE seminar_id = '$delete_id';");
				if ($db2->affected_rows())
				{
					$info['delete'] = true;
				}
			}
			//wenn nicht, titel info aus der seminare holen, da es wohl probleme gibt
			else
			{
				$db2 = new DB_Seminar;
				$db2->query("SELECT name FROM seminare WHERE Seminar_id = '$delete_id';");
				if($db->next_record())
				{
					$info['name'] = $db->f("name");
				}
			}
			log_event("SEM_DELETE_FROM_ARCHIVE",$delete_id,NULL,$db->f("name")." (".$db->f("semester").")");
			//dateien lschen
			if ($db->f("archiv_file_id"))
			{
				if (@unlink($GLOBALS['ARCHIV_PATH']."/".$db->f("archiv_file_id")))
				{
					$info['file'] = true;
				}
				else
				{
					$info['fileerror'] = true;
				}
			}
			//teilnehmer lschen
			$db2->query("DELETE FROM archiv_user WHERE seminar_id='$delete_id';");
			if ($db2->affected_rows())
			{
				$info['user'] = $db2->affected_rows();
			}
			array_push($result, $info);
		}
		return $result;
	}

	/**
	 * Gibt die Anzahl Semester zurck, in denen es archivierte Veranstalungen gibt
	 *
	 * @return int
	 */
	public static function countSemester()
	{
		$db = new DB_Seminar;
		$db->query("SELECT COUNT(DISTINCT semester) AS count FROM archiv;");
		if ($db->next_record())
		{
			return ($db->f('count'));
		}
	}

	/**
	 * Gibt die anzahl archivierter Veranstaltungen zurck
	 *
	 * @return int
	 */
	public static function countCourses()
	{
		$db = new DB_Seminar;
		$db->query("SELECT COUNT(*) AS count FROM archiv;");
		if ($db->next_record())
		{
			return ($db->f('count'));
		}
	}

	/**
	 * Gibt die Anzahl archivierter Veranstaltungen mit Dateien zurck
	 *
	 * @return int
	 */
	public static function countFiles()
	{
		$db = new DB_Seminar;
		$db->query("SELECT COUNT(archiv_file_id) AS files FROM archiv WHERE archiv_file_id != '';");
		if ($db->next_record())
		{
			return ($db->f('files'));
		}
	}

	/**
	 * Enter description here...
	 *
	 * @return int
	 */
	public static function countBoards()
	{
		$db = new DB_Seminar;
		$db->query("SELECT COUNT(seminar_id) AS boards FROM archiv WHERE forumdump != '';");
		if ($db->next_record())
		{
			return ($db->f('boards'));
		}
	}

	/**
	 * Enter description here...
	 *
	 * @return int
	 */
	public static function countWikis()
	{
		$db = new DB_Seminar;
		$db->query("SELECT COUNT(seminar_id) AS wikis FROM archiv WHERE wikidump != '';");
		if ($db->next_record())
		{
			return ($db->f('wikis'));
		}
	}
	/**
	 * Gibt das Datum der letzten Archivierung zurck
	 *
	 * @return string
	 */
	public static function getLastArchivDate()
	{
		$db = new DB_Seminar;
		$db->query("SELECT MAX(mkdate) AS maxdate FROM archiv;");
		if ($db->next_record())
		{
			return (date("d.m.Y", $db->f('maxdate')));
		}
	}

    /**
     * Gibt Informationen ber den Archivordner zurck
     *
     * @return array
     */
	public static function getArchiveFolderInfo()
	{
		$folder_size = 0;
		$files_in_folder = 0;
		$dir_name = $GLOBALS['ARCHIV_PATH'];
		if (is_dir($dir_name)) {
			if ($dh = opendir($dir_name)) {
				while (($file = readdir($dh)) !== false) {
					if ($file != '.' && $file != '..') {
						if(is_file($dir_name . '/' . $file)) {
							$folder_size += filesize($dir_name . '/' . $file);
							++$files_in_folder;
						}
					}
				}
				closedir($dh);
			}
		}
		return compact('folder_size', 'files_in_folder');
	}


}